/*!
 * @author Cindy Rubio Gonzalez
 */

#include "ErrorPropagation.hpp"
#include "fdd.h"
#include <algorithm>

extern int counter;
extern map<string, int> table;
extern int numberVars;

ErrorPropagation::ErrorPropagation( bdd b) : BDD(b), file("") {}

ErrorPropagation::ErrorPropagation( bdd b, set<int> l, string f) : BDD(b), lines(l), file(f) {}

ErrorPropagation::ErrorPropagation(sem_elem_t s) {
  ErrorPropagation* t = static_cast<ErrorPropagation*>(s.get_ptr());
  BDD = t->BDD;
  lines = t->lines;
  file = t->file;
}


ErrorPropagation::~ErrorPropagation() {
}


sem_elem_t ErrorPropagation::one() const {
  bdd ONE = bddfalse;
  for(int i = 0; i < numberVars; i++) {
	ONE |= (fdd_ithvar(0, i) & fdd_ithvar(1, i));
  }
  return new ErrorPropagation(ONE);
}


sem_elem_t ErrorPropagation::zero() const {
  return new ErrorPropagation(bddfalse);
}


sem_elem_t ErrorPropagation::extend( SemElem* se ) {

  ErrorPropagation* rhs = static_cast< ErrorPropagation* >(se);
  
  // 1. Shifting down bdd BDD1 dom 0 -> dom 1, dom 1 -> dom 2 
  bddPair *p = bdd_newpair();
  int i;
  for(i = 0;  i < 2; i++) {
	fdd_setpair(p, i, i+1); 
  }
   
  bdd bdd1 = bdd_replace(BDD, p);

  // 2. Performing relational product 
  bdd result = bdd_relprod(bdd1, rhs->BDD, fdd_ithset(1));

  // 3. Shifting up dom 2 -> dom 1 in result 
  bddPair *p2 = bdd_newpair();
  fdd_setpair(p2, 2, 1);

  // source info
  set<int>::iterator b1 = lines.begin();
  set<int>::iterator e1 = lines.end();
  set<int>::iterator b2 = rhs->lines.begin();
  set<int>::iterator e2 = rhs->lines.end();

  set<int> union_set;
  set_union(b1, e1, b2, e2, inserter(union_set, union_set.begin()) );
  
  string s;
  if (file == "")
	s = rhs->file;
  else
	s = file;

  return new ErrorPropagation( bdd_replace(result, p2), union_set, s); 
}


sem_elem_t ErrorPropagation::combine( SemElem* se ) {
  ErrorPropagation* rhs = static_cast< ErrorPropagation* >(se);

  // source info
  set<int> the_lines;
  if (lines.size() < rhs->lines.size())
	the_lines = lines;
  else
	the_lines = rhs->lines;

  return new ErrorPropagation( bdd_or(BDD, rhs->BDD), the_lines, file); 
}


bool ErrorPropagation::equal( SemElem* se ) const {
    ErrorPropagation* rhs = static_cast< ErrorPropagation* >(se);
	return BDD == rhs->BDD;
}


std::ostream & ErrorPropagation::print( std::ostream & o ) const {
  o << std::endl << fddset << BDD << std::endl;
  o << "source file: " << file << std::endl;
  o << "line numbers: ";

  for(set<int>::iterator it = lines.begin(); it != lines.end(); it++) {
	o << (*it) << " ";
  }
  o << std::endl;
  return o;
}


sem_elem_t ErrorPropagation::getWeight( std::string& s ) {
  if (s == "ONE")
	return one();
  else
	return zero();
}


sem_elem_t ErrorPropagation::getWeight( std::string& s1, std::string& s2 ) {
  std::cerr << "SHOULD NOT CALL THIS GETWEIGHT FUNCTION!!!!\n";

  if (s1 != "" && table.find(s1) == table.end()) {
	table[s1] = counter++;
  }
  if (s2 != "" && table.find(s2) == table.end()) {
	table[s2] = counter++;
  }
  
  bdd BDD1 = bddfalse;
  
  if ( ( s1 == "") && ( s2 == "" ) ) {
	for(int i = 0; i < numberVars; i++) {
	  BDD1 |= (fdd_ithvar(0, i) & fdd_ithvar(1, i));
	}
  }
  else {
	int i1 = table[s1];
	int i2 = table[s2];

	for(int i = 0; i < numberVars; i++) {
	  if (i != i1)
		BDD1 |= (fdd_ithvar(0, i) & fdd_ithvar(1, i));
	}
	
	BDD1 |= (fdd_ithvar(0, i1) & fdd_ithvar(1, i2));
  }
  
  return new ErrorPropagation(BDD1);
}



sem_elem_t ErrorPropagation::getWeight( std::string& s1, std::string& s2, int line, string f ) {

  if (s1 != "" && table.find(s1) == table.end()) {
	table[s1] = counter++;
  }
  if (s2 != "" && table.find(s2) == table.end()) {
	table[s2] = counter++;
  }
  
  bdd BDD1 = bddfalse;
  
  if ( ( s1 == "") && ( s2 == "" ) ) {
	for(int i = 0; i < numberVars; i++) {
	  BDD1 |= (fdd_ithvar(0, i) & fdd_ithvar(1, i));
	}
  }
  else {
	int i1 = table[s1];
	int i2 = table[s2];

	for(int i = 0; i < numberVars; i++) {
	  if (i != i1)
		BDD1 |= (fdd_ithvar(0, i) & fdd_ithvar(1, i));
	}
	
	BDD1 |= (fdd_ithvar(0, i1) & fdd_ithvar(1, i2));
  }
  
  set<int> lines1 = set<int>();
  lines1.insert(line);

  return new ErrorPropagation(BDD1, lines1, f);
}

/* Yo, Emacs!
   ;;; Local Variables: ***
   ;;; tab-width: 4 ***
   ;;; End: ***
*/
