
/*!
 * @author Nick Kidd, Modified by Cindy Rubio Gonzalez
 */

#include "wali/QueryHandler.hpp"
#include "wali/wpds/WpdsHandler.hpp"
#include "wali/wfa/WfaHandler.hpp"
#include "StrX.hpp"
#include "ErrorPropagation.hpp"
#include "MergeError.hpp"
#include "TransQuery.hpp"

#include "ParseArgv.hpp"
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/TransService.hpp>
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/util/OutOfMemoryException.hpp>
#include "fdd.h"
#include "wali/wpds/ewpds/EWpdsHandler.hpp"
#include "wali/wpds/ewpds/EWPDS.hpp"        
#include "wali/witness/WitnessWrapper.hpp"


XERCES_CPP_NAMESPACE_USE

map<string, int> table;
int counter;

int parseWpds( std::string& xmlFile );
int parseWfa( std::string& xmlFile );
int parseQuery( std::string& xmlFile );
int parseFile( DefaultHandler&, std::string& xmlFile );
void print_table();

int main( int argc, char** argv ) {
    counter = 0;

    k::ParseArgv parser(argc,argv);
    std::string fname;

    int rc=0;
    if( parser.get("--wpds",fname) ) {
        rc |= parseWpds(fname);
    }
    if( parser.get("--wfa",fname) ) {
        rc |= parseWfa(fname);
    }
    if( parser.get("--query",fname) ) {
        rc |= parseQuery(fname);
    }

    print_table();

    return rc;
}

int parseWpds( std::string& xmlFile )
{
  wali::wpds::ewpds::EWPDS *ewpds = new wali::wpds::ewpds::EWPDS(); 
    ErrorPropagation wf(bddfalse);                                     
    MergeError mf;
    wali::wpds::ewpds::EWpdsHandler handler(ewpds, wf, mf);       
    
    int rc = parseFile(handler,xmlFile);
    handler.get().print( std::cout );
    return rc;
}

int parseWfa( std::string& xmlFile )
{
    ErrorPropagation wf(bddfalse);
    wali::wfa::WfaHandler handler(wf);
    int rc = parseFile(handler,xmlFile);
    handler.get().print( std::cout );
    return rc;
}

int parseQuery( std::string& xmlFile )
{

  wali::witness::WitnessWrapper* wrapper = new wali::witness::WitnessWrapper();

    wali::wpds::ewpds::EWPDS *ewpds = new wali::wpds::ewpds::EWPDS(wrapper); 
    ErrorPropagation wf(bddfalse);
    MergeError mf;

    wali::wpds::ewpds::EWpdsHandler ewpdsHandler(ewpds, wf, mf);       
    wali::wfa::WfaHandler wfaHandler(wf);                              

    wali::QueryHandler handler(&ewpdsHandler, &wfaHandler);            

    int rc = parseFile(handler,xmlFile);
    if( 0 == rc ) {
        std::cout << "Query Result\n";
        std::cout << "----------------------------------------\n";
	
	const wali::wfa::WFA result = handler.run();
	result.print(std::cout);

	TransQuery query(std::cout);
	result.for_each(query);
	
    }
    return rc;
}

int parseFile( DefaultHandler& handler,std::string& xmlFile )
{
    int errorCode = 0;

    // Initialize the XML4C2 system
    try {    
        XMLPlatformUtils::Initialize();
    }   
    catch (const XMLException& toCatch) {       
        XERCES_STD_QUALIFIER cerr << "Error during initialization! :\n" << StrX(toCatch.getMessage()) << XERCES_STD_QUALIFIER endl;
        return 1; 
    }

    SAX2XMLReader* parser = XMLReaderFactory::createXMLReader();


    try {
        parser->setContentHandler(&handler);
        parser->setErrorHandler(&handler);
        parser->parse(xmlFile.c_str());
    }
    catch( const OutOfMemoryException&)
    {
        XERCES_STD_QUALIFIER cerr << "OutOfMemoryException" << XERCES_STD_QUALIFIER endl;
        errorCode = 2;
    }
    catch (const XMLException& toCatch)
    {
        XERCES_STD_QUALIFIER cerr << "\nAn error occurred\n  Error: "
            << StrX(toCatch.getMessage())
            << "\n" << XERCES_STD_QUALIFIER endl;
        errorCode = 3;
    }
    catch( const SAXException& toCatch ) {
        std::cerr << "Error in Main: " << StrX(toCatch.getMessage()) << std::endl;
        errorCode = 4;
    }

    delete parser;

    XMLPlatformUtils::Terminate();

    return errorCode;
}


void print_table() {
  std::cerr << std::endl << "===== Hash Table =====" << std::endl;

  map<string, int>::iterator it = table.begin();
  for( ; it != table.end(); it++) {
    std::cerr << (*it).second << " " << (*it).first << std::endl;
  }

  return;
}


