/*!
 * @author Cindy Rubio Gonzalez
 */

#include "MergeError.hpp"
#include "ErrorPropagation.hpp"
#include "fdd.h"
#include "wali/ref_ptr.hpp"
#include <vector>

using namespace std;

extern int numberVars;
extern vector<string> nameVars;
extern map<string, int> table;

MergeError::MergeError() {
}

merge_fn_t MergeError::getMergeFn( std::string& s ) {
  return new MergeError();
}



SemElem* MergeError::apply_f(SemElem* w1, SemElem* w2) {

  //Procedure using global variables
  // G -> U and G -> I
  int index;
  bdd GU = bddfalse;
  bdd GI = bddfalse;

  vector<string>::iterator it = nameVars.begin();
  for(; it != nameVars.end(); it++) {
	index = table[*it];
	for(int i = 0; i < numberVars; i++) {
	  GU |= (fdd_ithvar(0, index) & fdd_ithvar(1, i));
	}
	GI |= (fdd_ithvar(0, index) & fdd_ithvar(1, index));
  }

  // E -> I
  bdd EI = bddfalse;
  for(int i = 0; i < numberVars; i++) {
	EI |= (fdd_ithvar(0, i) & fdd_ithvar(1, i));
  }

  // L -> I
  bdd LI = EI - GI;

  ErrorPropagation* ew1 = static_cast<ErrorPropagation*>(w1);
  ErrorPropagation* ew2 = static_cast<ErrorPropagation*>(w2);

  ErrorPropagation* ew2_new = new ErrorPropagation( bdd_and(GU, ew2->BDD), ew2->lines, ew2->file );
  ew2_new->BDD |= LI;
 
  ErrorPropagation* result = new ErrorPropagation(ew1->extend(ew2_new));
  std::cout << "result: "; result->print(std::cout);

  return result; 

  /*
	//Default implementation where extend is performed
  sem_elem_t r = w1->extend(w2);
  return new ErrorPropagation(r);
  */
}

/* Yo, Emacs!
   ;;; Local Variables: ***
   ;;; tab-width: 4 ***
   ;;; End: ***
*/
