#ifndef ERROR_PROPAGATION_GUARD
#define ERROR_PROPAGATION_GUARD 1

/*!
 * @author Cindy Rubio Gonzalez
 */

#include "wali/SemElem.hpp"
#include "wali/WeightFactory.hpp"
#include <string>
#include "fdd.h"
#include <map>
#include "wali/wpds/ewpds/MergeFunction.hpp"
#include "wali/SemElemPair.hpp"
#include <vector>
#include "wali/wpds/WeightInfo.hpp"
#include <queue>
#include <fstream>


using namespace std;

using wali::SemElem;
using wali::SemElemPair;
using wali::sem_elem_t;
using wali::wpds::ewpds::merge_fn_t;

class ErrorPropagation : public wali::SemElem, public wali::WeightFactory {

public:
  ErrorPropagation();
  ErrorPropagation( bdd b);
  ErrorPropagation( bdd b, int l, string f, bool t, string fn );
  ErrorPropagation( sem_elem_t s);
  
  
  virtual ~ErrorPropagation();
  
  sem_elem_t one() const;
  
  sem_elem_t zero() const;
  
  // zero is the annihilator for extend
  sem_elem_t extend( SemElem* rhs );
  
  // zero is neutral for combine
  sem_elem_t combine( SemElem* rhs );
  
  bool equal( SemElem* rhs ) const;
  
  std::ostream & print( std::ostream & o ) const;
  
  // For WeightFactory
  sem_elem_t getWeight( std::string& s );
  virtual sem_elem_t getWeight( queue<WeightInfo>& weightInfos, int line, string file );

  virtual void printDot(ofstream& fout, ofstream& foutAll) const;
  string getFunction();

  bool hasError(const int var, const int error);
  string hasError(int& var, int &error, bool& slice);
  bdd BDD;  

protected:
  int line_number;
  string file_name;
  bool is_trusted;
  string function_name;
  
  bdd extend(const bdd &BDD1, const bdd &BDD2);
  string receivesErrorMsg(int target, int source, int &error);
  string mayHaveErrorMsg(int target, int &error);

  };

#endif	// ERROR_PROPAGATION_GUARD

/*
 * $Log $
 */

/* Yo, Emacs!
;;; Local Variables: ***
;;; tab-width: 4 ***
;;; End: ***
*/

