/*!
 * @author Cindy Rubio Gonzalez
 */

#include "ErrorPropagationPopRule.hpp"
#include "fdd.h"
#include "Variables.hpp"
#include <stack>

extern int numberVars;
extern bdd identity;
extern bdd globalsToIdentity;
extern bdd allToUninitialized;
extern bdd localsToUninitialized;
extern stack<string> callstack;


ErrorPropagationPopRule::ErrorPropagationPopRule( bdd b, int l, string f, bool t, string fn) {
  BDD = b;
  line_number = l;
  file_name = f;
  is_trusted = t;
  function_name = fn;
}


sem_elem_t ErrorPropagationPopRule::getWeight(queue<WeightInfo>& weightInfos, int line, string file) {

  bdd result = identity;
  bdd weight;
  bool trusted = true;
  string function = "";
  

  // Adding given variable relationships
  while (! weightInfos.empty()) {
	WeightInfo w = weightInfos.front();

	trusted = trusted && w.trusted;
	function = w.function;

	if ((w.weight1 == "identity") && (w.weight2 == "identity")) {
	  weight = identity;
	}
	else if ((w.weight1 == "identityGlobals") && (w.weight2 == "identityGlobals")) {
	  weight = globalsToIdentity | localsToUninitialized;
	}
	else if ((w.weight1 == "uninitialized") && (w.weight2 == "uninitialized")) {
	  weight = allToUninitialized;
	}

	else if ((w.weight1 != "") && (w.weight2 != "")) {

	  variables->add(w.weight1);
	  variables->add(w.weight2);
	
	  int i1 = variables->getIndex(w.weight1);
	  int i2 = variables->getIndex(w.weight2);

	  weight = identity;
	  
	  weight -= (fdd_ithvar(0, i1) & fdd_ithvar(1, i1)); //removing the mapping to itself from identity bdd
	  weight |= (fdd_ithvar(0, i1) & fdd_ithvar(1, i2)); //adding new relation
	}
	else {
	  assert(false);
	}
	
	result = extend(result, weight);
	weightInfos.pop();
  }

  return new ErrorPropagationPopRule(result, line, file, trusted, function);
}


void ErrorPropagationPopRule::printDot(ofstream& fout, ofstream& foutAll) const {

  /*delete*///std::cerr << "DOT: ErrorPropationPopRule -- function: " 
  //	  << function_name << std::endl;


  
  if (callstack.top() == function_name) {//there was a push rule
	callstack.pop();
	
	/*
	if (! callstack.empty()) {
	  fout << function_name << "->" << callstack.top() 
		   << "[style=\"setlinewidth(1)\" arrowsize=1];" << endl;
	  foutAll << function_name << "->" << callstack.top() 
		   << "[style=\"setlinewidth(1)\" arrowsize=1];" << endl;

	}
	else {
	  fout << function_name << "->" << function_name 
		   << "[style=\"setlinewidth(1)\" arrowsize=1];" << endl;
	  foutAll << function_name << "->" << function_name 
			  << "[style=\"setlinewidth(1)\" arrowsize=1];" << endl;
	}
	*/
  }
  else { //add push first
	fout << "  " << function_name << "[ label=\"" << function_name 
		 << "\"];" << endl;
	fout << "  " << callstack.top() << "->" << function_name 
		 << "  " << "[style=\"setlinewidth(1)\" arrowsize=1 ];" 
		 << endl;
	/*
	fout << "  " << function_name << "->" << callstack.top() 
		 << "[style=\"setlinewidth(1)\" arrowsize=1];" << endl;
	*/

	foutAll << "  " << function_name << "[ label=\"" << function_name 
			<< "\"];" << endl;
	foutAll << "  " << callstack.top() << "->" << function_name 
			<< "  " << "[style=\"setlinewidth(1)\" arrowsize=1 ];" 
		 << endl;
	/*
	foutAll << "  " << function_name << "->" << callstack.top() 
			<< "[style=\"setlinewidth(1)\" arrowsize=1];" << endl;
	*/
  }
  
}


/* Yo, Emacs!
   ;;; Local Variables: ***
   ;;; tab-width: 4 ***
   ;;; End: ***
*/
