/*!
 * @author Cindy Rubio Gonzalez
 */

#include "ErrorPropagationPushRule.hpp"
#include "fdd.h"
#include "Variables.hpp"
#include <stack>

extern int numberVars;
extern bdd identity;
extern bdd globalsToIdentity;
extern bdd allToUninitialized;
extern bdd localsToUninitialized;
extern stack<string> callstack;


ErrorPropagationPushRule::ErrorPropagationPushRule( bdd b, int l, string f, bool t, string fn, string c) {
  BDD = b;
  line_number = l;
  file_name = f;
  is_trusted = t;
  function_name = fn;
  callee = c;
}


sem_elem_t ErrorPropagationPushRule::getWeight(queue<WeightInfo>& weightInfos, int line, string file) {

  bdd result = identity;
  bdd weight;
  bool trusted = true;
  string function = "";
  string the_callee;
  

  // Adding given variable relationships
  while (! weightInfos.empty()) {
	WeightInfo w = weightInfos.front();

	trusted = trusted && w.trusted;
	function = w.function;
	the_callee = w.callee;

	if ((w.weight1 == "identity") && (w.weight2 == "identity")) {
	  weight = identity;
	}
	else if ((w.weight1 == "identityGlobals") && (w.weight2 == "identityGlobals")) {
	  weight = globalsToIdentity | localsToUninitialized;
	}
	else if ((w.weight1 == "uninitialized") && (w.weight2 == "uninitialized")) {
	  weight = allToUninitialized;
	}

	else if ((w.weight1 != "") && (w.weight2 != "")) {

	  variables->add(w.weight1);
	  variables->add(w.weight2);
	
	  int i1 = variables->getIndex(w.weight1);
	  int i2 = variables->getIndex(w.weight2);

	  weight = identity;
	  
	  weight -= (fdd_ithvar(0, i1) & fdd_ithvar(1, i1)); //removing the mapping to itself from identity bdd
	  weight |= (fdd_ithvar(0, i1) & fdd_ithvar(1, i2)); //adding new relation
	}
	else {
	  assert(false);
	}
	
	result = extend(result, weight);
	weightInfos.pop();
  }

  return new ErrorPropagationPushRule(result, line, file, trusted, function, the_callee);
}


void ErrorPropagationPushRule::printDot(ofstream& fout, ofstream& foutAll) const {
  string from = function_name;
  string to = callee;

  /*delete*///std::cerr << "DOT: ErrorPropagationPushRule -- function: " 
  //	  << function_name << " !calling: " << callee << std::endl;

  fout << "  " << to << "[ label=\"" << to << "\"];" << endl;
  fout << from << "->" << to 
	   << "[style=\"setlinewidth(1)\" arrowsize=1];" << endl;
  
  foutAll << "  " << to << "[ label=\"" << to << "\"];" << endl;
  foutAll << from << "->" << to 
		  << "[style=\"setlinewidth(1)\" arrowsize=1];" << endl;

  callstack.push(to);

}

/* Yo, Emacs!
   ;;; Local Variables: ***
   ;;; tab-width: 4 ***
   ;;; End: ***
*/
