/*!
 * @author Cindy Rubio Gonzalez
 */

#include "MergeError.hpp"
#include "ErrorPropagation.hpp"
#include "fdd.h"
#include "wali/ref_ptr.hpp"
#include <vector>
#include "Variables.hpp"

using namespace std;

extern int numberVars;
extern vector<string> globalVars;

extern bdd globalsToUniverse;
extern bdd localsToIdentity;

extern bdd bddTentativeErrors;
extern map<int, int> mapTentativeToReal;
extern vector<string> tentativeErrorNames;


MergeError::MergeError(std::string f) {
  function = f;
}

merge_fn_t MergeError::getMergeFn( std::string& f) {
  return new MergeError(f);
}



SemElem* MergeError::apply_f(SemElem* w1, SemElem* w2) {

  ErrorPropagation* ew1 = static_cast<ErrorPropagation*>(w1);
  ErrorPropagation* ew2 = static_cast<ErrorPropagation*>(w2);

  ErrorPropagation ew2_new( bdd_and(globalsToUniverse, ew2->BDD) ); //before calling the other constructor
  ew2_new.BDD |= localsToIdentity;


  //////////////////new
  int index = variables->getIndex(function + "$return");
  bdd bddVarToTentative = fdd_ithvar(0, index) & bddTentativeErrors;

  if ((ew2_new.BDD & bddVarToTentative) != bddfalse) {

	for(int unsigned i = 0; i < tentativeErrorNames.size(); i++) {

	  int indexTentative = variables->getIndex(tentativeErrorNames[i]);
	  bdd varToTentative = fdd_ithvar(0, index) & fdd_ithvar(1, indexTentative);

	  if ((ew2_new.BDD & varToTentative) != bddfalse) {
		ew2_new.BDD -= varToTentative;
		int indexReal = mapTentativeToReal[indexTentative];
		bdd varToReal = fdd_ithvar(0, index) & fdd_ithvar(1, indexReal);
		ew2_new.BDD |= varToReal;
	  }
	}
	
  }
  //////////////////till here

  ErrorPropagation* result = new ErrorPropagation(ew1->extend(&ew2_new));
  //std::cout << "result: "; result->print(std::cout);

  return result; 
  
  
  //Default implementation where extend is performed
  /*
  sem_elem_t r = w1->extend(w2);
  return new ErrorPropagation(r);
  */
}

/* Yo, Emacs!
   ;;; Local Variables: ***
   ;;; tab-width: 4 ***
   ;;; End: ***
*/
