#include "ProgramPoint.hpp"
#include <iostream>
using namespace std;

extern bdd bddErrors;
extern map<int, bdd> mapErrorToBdd;

ProgramPoint::ProgramPoint(string f, string func, int l, map<string, string> t, string m, wali::Key fk) {
    file = f;
    function = func;
    line = l;
    targets = t;
    msgcode = m;
    fromStkKey = fk;
    weight = NULL;
 }


void ProgramPoint::calculateWeight(const wali::wfa::WFA& result, const map<wali::Key, wali::wfa::Trans*>& mapTrans) {

  map<wali::Key, wali::wfa::Trans*>::const_iterator it = mapTrans.find(fromStkKey);

  if (it != mapTrans.end()) {
    wali::wfa::Trans* t = it->second;

    weight_for_fromStkKey = result.getState(t->to())->weight()->extend(t->weight());
    witness = static_cast<wali::witness::Witness*>( weight_for_fromStkKey.get_ptr() );
    weight = static_cast<const ErrorPropagation*>( witness->weight().get_ptr() );
  }
}


//void ProgramPoint::processTargets(const int error) {
void ProgramPoint::processTargets() {
  
  if (weight != NULL) {
    
    map<string, string>::iterator it = targets.begin();
    for(; it != targets.end(); it++) {
      
      int target = variables->getIndex(it->first);
      int source = variables->getIndex(it->second);
      bdd targetToErrors = fdd_ithvar(0, target) & bddErrors;

      if ( (weight->BDD & targetToErrors) != bddfalse ) { //the var has at least one error
      
	for(unsigned int i = 0; i < errorNames.size(); i++) {
	  const int error = variables->getIndex(errorNames[i]);
	  
	  if (interesting(target, source, error)) {
	    
	    bool tentative = true;
	    if (i < 34) {
	      tentative = false;
	    }

	    FindPathVisitor path(target, error);
	    witness->accept(path);
	    
	    path.printPath(std::cout, file, line, it->first, msgcode, tentative);
	    std::cout << std::endl;
	    
	    path.printSlice(std::cout, file, line, it->first, msgcode, tentative);
	    std::cout << "====\n";
	    
	    break;  //just want to complain about one error code

	  }
	}
      }
    }
  }
}



bool ProgramPoint::interesting(int target, int source, int error) {

  bdd targetToError = (fdd_ithvar(0,target) & (fdd_ithvar(1,error)));
  if ( (weight->BDD & targetToError) != bddfalse ) {

    bdd sourceToAllButError = fdd_ithvar(0, source) & mapErrorToBdd[error];
    if ( (weight->BDD & sourceToAllButError) != bddfalse) {
      return true;
    }
    
    return false;
  }

  return false;
}

