#include "Variables.hpp"
#include <iostream>
#include <fstream>
#include "fdd.h"

using namespace std;
vector<string> errorNames; //has all errors include tentative errors
vector<string> tentativeErrorNames;
int Variables::numVars = 0;

auto_ptr<Variables> variables;


Variables::Variables(const string &errorCodes) {

  // Adding OK and uninitialized
  add("OK");
  add("UNINITIALIZED");

  // Reading error names from file
  ifstream inFile(errorCodes.c_str());
  string name;
  int value;

  if (!inFile) {
    std::cerr << "Unable to open " << errorCodes << '\n';
    exit(1);
  }

  while(inFile >> name >> value) {
    errorNames.push_back(name);
    add(name);
    tentativeErrorNames.push_back("TENTATIVE_" + name);
    add("TENTATIVE_" + name);
  }

  for(unsigned int i = 0; i < tentativeErrorNames.size(); i++) {  //we want the tentative errors to be at the end of all errors.
    errorNames.push_back(tentativeErrorNames[i]);
  }
 
}


void Variables::add(const string &v) {
  if (!v.empty() && vars.find(v) == vars.end()) {
    vars[v] = numVars++;
    names.push_back(v);
  }
}

int Variables::getIndex(const string &v) {
  return vars[v];
}

string Variables::getName(unsigned int i) {
  if (i < names.size())
    return names[i];
  else
    return "[ERROR] Variable unitialized!\n";
}

void Variables::print( std::ostream& out) {
  for(unsigned int i = 0; i < names.size(); i++) {
    out << i << " " << names[i] << std::endl;
  }
}
