from SCons.Builder import Builder
from SCons.Scanner import Scanner


########################################################################
#
#  run error detection propagation analysis on a weighted pushdown system
#


__edp_action = [['$__VALGRIND', '$EDP', '$EDP_FLAGS', '--error-codes=$ERROR_CODES', '--query=$SOURCE', '>$TARGET']]


def __edp_target_scanner(node, env, path):
    dependencies = [env['EDP'], env['ERROR_CODES']]
    if env['VALGRIND']:
        dependencies.append(env['VALGRIND_SUPPRESSIONS'])
    return dependencies


def __edp_emitter(target, source, env):
    if env['VALGRIND']:
	[target0] = target
        log = target0.target_from_source('', '.log')
        env.Clean(target0, log)
    return target, source


__edp_builder = Builder(
    action=__edp_action,
    src_suffix='.wpds',
    src_builder='WPDS',
    suffix='.out',
    target_scanner=Scanner(__edp_target_scanner),
    single_source=True,
    emitter=__edp_emitter,
    )


########################################################################


def __var_valgrind(target, source, env, for_signature):
    if env['VALGRIND']:
        return ['valgrind', '--tool=memcheck', '--leak-check=full', '--suppressions=$VALGRIND_SUPPRESSIONS', '--log-file-exactly=${TARGET.base}.log']
    else:
        return []


def generate(env):
    env.AppendUnique(
        EDP=env.File('#/error-intra/EDP-intra'),
        BUILDERS={'EDP': __edp_builder},
        VALGRIND_SUPPRESSIONS=env.File('valgrind.supp'),
        __VALGRIND=__var_valgrind,
        )


def exists(env):
    return True
