/*!
 * @author Nick Kidd
 */

#include "wali/Common.hpp"
#include "wali/wpds/WpdsHandler.hpp"
#include "StrX.hpp"
#include "wali/WeightFactory.hpp"
#include <xercesc/sax2/Attributes.hpp>
#include "wali/Key.hpp"
#include "wali/wpds/WPDS.hpp"
#include "wali/wpds/Rule.hpp"
#include <fdd.h>
#include "wali/witness/WitnessWrapper.hpp"
#include "../../error-intra/Variables.hpp" //Should get rid of this relative path
#include <queue>
#include <vector>

using namespace std;

int numberVars;
bdd identity;
bdd globalsToUniverse;
bdd localsToIdentity;
bdd globalsToIdentity;
bdd allToUninitialized;
bdd localsToUninitialized;

bdd bddErrors;
bdd bddErrorsOK;
map<int, bdd> mapErrorToBdd;  
bdd bddTentativeErrors;
map<int, int> mapTentativeToReal;
map<int, int> mapRealToTentative;

vector<string> globalVars;

queue<WeightInfo>  weightInfos;
map<string, string> targets;
void populatingErrorDS();

namespace wali
{
    namespace wpds
    {

	   WpdsHandler::WpdsHandler( WeightFactory& wf, WPDS* thePds ) :
            weightFactory(wf), inWeight(false),useWeight(false),pds(thePds)
        {
		  // I added the witness wrapper parameter
		  
		  if( 0 == pds ) {
			wali::witness::WitnessWrapper* wrapper = new wali::witness::WitnessWrapper();
			pds = new WPDS(wrapper);

		  }

		  // Initialize the XML4C2 system
		  // Necessary for using transcode
		  XMLPlatformUtils::Initialize();
		  fromStackID = XMLString::transcode(Rule::XMLFromStackTag.c_str());
		  fromID = XMLString::transcode(Rule::XMLFromTag.c_str());
		  toID = XMLString::transcode(Rule::XMLToTag.c_str());
		  toStack1ID = XMLString::transcode(Rule::XMLToStack1Tag.c_str());
		  toStack2ID = XMLString::transcode(Rule::XMLToStack2Tag.c_str());
		  
		  loc1ID = XMLString::transcode("loc");
		  loc2ID = XMLString::transcode("loc");
		  trustedID = XMLString::transcode("trusted");
		  nameID = XMLString::transcode("name");
		  globalsID = XMLString::transcode("globals");
		  localsID = XMLString::transcode("locals");
		  
		  lineID = XMLString::transcode("line");
		  fileID = XMLString::transcode("file");
		  
		  inSet = false;
		  inIdentity = false;
        }

        WpdsHandler::~WpdsHandler()
        {
		  XMLString::release(&fromID);
		  XMLString::release(&fromStackID);
		  XMLString::release(&toID);
		  XMLString::release(&toStack1ID);
		  XMLString::release(&toStack2ID);
		  XMLString::release(&loc1ID);
		  XMLString::release(&loc2ID);
		  XMLString::release(&trustedID);
		  XMLString::release(&globalsID);
		  XMLString::release(&localsID);
		  XMLString::release(&nameID);
		  XMLString::release(&lineID);
		  XMLString::release(&fileID);
		  XMLPlatformUtils::Terminate();
		  
		  if( 0 == pds ) {
			assert(0);
		  }
		  delete pds;
		  
        }

        WPDS& WpdsHandler::get() {
            return *pds;
        }

        //////////////////////////////////////////////////
        // Parsing handlers
        //////////////////////////////////////////////////

        void WpdsHandler::startDocument()
        {
        }

        void WpdsHandler::endDocument()
        {
        }

	  void WpdsHandler::startElement(  const   XMLCh* const    uri,
									   const   XMLCh* const    localname,
									   const   XMLCh* const    qname,
									   const   Attributes&     attributes)
	  {
		StrX who(localname);
		
		if (strcmp(who.get(), "WPDS") == 0) {
		  //handleWPDS(uri, localname, qname, attributes);
		}
		else if (strcmp(who.get(), "var") == 0) {
		  handleVar(uri, localname, qname, attributes);
		}
		else if (strcmp(who.get(), "Variables") == 0) {
		  handleVariables(uri, localname, qname, attributes);
		}
		else if (Rule::XMLTag == who.get()) {
		  weightString1 = "";
		  weightString2 = "";
		  handleRule(uri, localname, qname, attributes);
		}
		else if (SemElem::XMLTag == who.get()) {
		  inWeight = true;
		  weightString = "";
		}
		else if (strcmp(who.get(),"set") == 0) {
		  inSet = true;
		  handleSet(uri, localname, qname, attributes);
		}
		else if (strcmp(who.get(), "from") == 0) {
		  handleFrom(uri, localname, qname, attributes);
		}
		else if ( (strcmp(who.get(), "identity") == 0) || 
				 (strcmp(who.get(), "unimplemented") == 0) ) {
		  inIdentity = true;
		}
		else if (strcmp(who.get(), "identityGlobals") == 0) {
		  
		}
		else if (strcmp(who.get(), "source") == 0) {
		  handleSource(uri, localname, qname, attributes);
		}
													   
		
		
		else {
		  /*
			std::cout << "<" << who << ">\n";
			for( unsigned i = 0 ; i < attributes.getLength() ; i++ ) {
			StrX lname(attributes.getLocalName(i));
			StrX val(attributes.getValue(i));
			std::cout << "\t" << lname << " -> " << val << std::endl;
			}
		  */
		}
	  }
	  

	  void WpdsHandler::endElement( const XMLCh* const uri,
									const XMLCh* const localname,
									const XMLCh* const qname)
	  {
		using wali::Key;
		StrX who(localname);
		
		
		if( SemElem::XMLTag == who.get() ) {
		  inWeight = false;
		}
		
		else if( Rule::XMLTag == who.get() ) {
		  
		  Key fromKey = getKey(from.get());
		  Key fromStkKey = getKey(fromStack.get());
		  Key toKey = getKey(to.get());
		  Key toStk1Key = getKey(toStack1.get());
		  Key toStk2Key = getKey(toStack2.get());

		  if (inSet || inIdentity) {
			//std::cout << "!!NO - should not be called when extended wpds is used..." << std::endl;
			/*pds->add_rule(fromKey
						  , fromStkKey
						  , toKey
						  , toStk1Key
						  , toStk2Key
						  , weightFactory.getWeight(weightString1, weightString2, line, file, trustedString)
						  );
			*/

			inSet = false;
			inIdentity = false;
		  }
		  else {
			//std::cout << "!!NO - calling weightString with one argument..." << std::endl;
			pds->add_rule(fromKey
						  , fromStkKey
						  , toKey
						  , toStk1Key
						  , toStk2Key
						  , weightFactory.getWeight(weightString)
						  );
			
		  } 

		} 
		else if (strcmp(who.get(),"set") == 0) {
		  weightString1 = loc1.get();
		  weightString2 = loc2.get();

		  trustedString = trusted.get();
		  useWeight = false; //??? the way it was...

		  /* Creating weightInfo */
		  WeightInfo wi;
		  wi.weight1 = weightString1;
		  wi.weight2 = weightString2;
		  wi.function = function;
		  wi.callee = callee;
		  
		  wi.trusted = false;
		  if (strcmp(trustedString.c_str(), "true") == 0) {
			wi.trusted = true;
		  }
		  else {
			targets[weightString1] = weightString2;
		  }

		  weightInfos.push(wi);
		  /*Done creating weightInfo */

		}
		else if ( (strcmp(who.get(), "identity") == 0) || 
				 (strcmp(who.get(), "unimplemented") == 0) ) {
		  weightString1 = "identity";
		  weightString2 = "identity";

		   /* Creating weightInfo */
		  WeightInfo wi;

		  wi.weight1 = weightString1;
		  wi.weight2 = weightString2;
		  wi.function = function;
		  wi.callee = callee;
		  wi.trusted = true;

		  weightInfos.push(wi);
		  /*Done creating weightInfo */

		}
		else if (strcmp(who.get(), "identityGlobals") == 0) {
		  weightString1 = "identityGlobals";
		  weightString2 = "identityGlobals";

		   /* Creating weightInfo */
		  WeightInfo wi;

		  wi.weight1 = weightString1;
		  wi.weight2 = weightString2;
		  wi.function = function;
		  wi.callee = callee;
		  wi.trusted = true;

		  weightInfos.push(wi);
		  /*Done creating weightInfo */
		}
		else if (strcmp(who.get(), "uninitialized") == 0) {
		  weightString1 = "uninitialized";
		  weightString2 = "uninitialized";

		   /* Creating weightInfo */
		  WeightInfo wi;

		  wi.weight1 = weightString1;
		  wi.weight2 = weightString2;
		  wi.function = function;
		  wi.callee = callee;
		  wi.trusted = true;

		  weightInfos.push(wi);
		  /*Done creating weightInfo */
		}

		else if (strcmp(who.get(), "Variables") == 0) {

		  /* Computing globals -> universe, locals -> identity to be used in apply_f MergeError */
		  /* Computing all -> uninitialized */

		  // G -> U and G -> I		  
		  bdd globalsBDD0 = bddfalse;
		  bdd globalsBDD1 = bddfalse;

		  globalsToIdentity = bddfalse;
		  for(int i = 0; i < globalVars.size(); i++) {
			globalsBDD0 |= fdd_ithvar(0, i);
			globalsBDD1 |= fdd_ithvar(1, i);
			globalsToIdentity |= (fdd_ithvar(0, i) & fdd_ithvar(1, i));
		  }

		  bdd allBDD = bddtrue;

		  int indexUninitialized = variables->getIndex("UNINITIALIZED");
		  bdd uninitializedBDD = fdd_ithvar(1, indexUninitialized);

		  globalsToUniverse = globalsBDD0 & allBDD;
		  allToUninitialized = allBDD & uninitializedBDD;
		  bdd globalsToUninitialized = globalsBDD0 & uninitializedBDD;

		  localsToIdentity = identity - globalsToIdentity;
		  localsToUninitialized = allToUninitialized - globalsToUninitialized;
		}
		
	  }


	  void WpdsHandler::characters(const XMLCh* const chars, const unsigned int length)
	  {
            StrX part(chars);
            if( inWeight) {
                weightString += part.get();
            }
            else {
                //std::cout << "Parse error?" << "\t\"" << part << "\" was seen.\n";
            }
        }

        void WpdsHandler::ignorableWhitespace(                               
                const XMLCh* const chars
                , const unsigned int length
                )
        {
        }

        void WpdsHandler::processingInstruction(   
                const XMLCh* const target
                , const XMLCh* const data
                )
        {
        }

        //////////////////////////////////////////////////
        // Default error handlers
        //////////////////////////////////////////////////

        void WpdsHandler::warning(const SAXParseException& exc)
        {
        }

        void WpdsHandler::error(const SAXParseException& exc)
        {
        }

        void WpdsHandler::fatalError(const SAXParseException& exc)
        {
        }

        //////////////////////////////////////////////////
        // Helpers
        //////////////////////////////////////////////////

      void WpdsHandler::handleVar(
                const XMLCh* const uri
                , const XMLCh* const localname
                , const XMLCh* const qname
                , const Attributes& attributes)
         {
		   sname = attributes.getValue(nameID);
		   globalVars.push_back(sname.get());
		   variables->add(sname.get()); //new
		}
		  


	  void WpdsHandler::handleVariables(
                const XMLCh* const uri
                , const XMLCh* const localname
                , const XMLCh* const qname
                , const Attributes& attributes)
        {
		   slocals = attributes.getValue(localsID);
		   sscanf(slocals.localForm(), "%d", &locals);

		   for(unsigned int i = 0; i < errorNames.size(); i++) {
			 globalVars.push_back(errorNames[i]);
		   } 
		   globalVars.push_back("OK");
		   globalVars.push_back("UNINITIALIZED");
		   numberVars = globalVars.size() + locals;

		   /* Needed before creating any BDD */
		   int domain[] = {numberVars,numberVars,numberVars};
		   bdd_init(100,100);
		   fdd_extdomain(domain, sizeof(domain)/sizeof(domain[0]));
		   bdd_gbc_hook(NULL);

		   /* Creating identity */
		   identity = bddfalse;
		   for(int i = 0; i < numberVars; i++) {
			 identity |= (fdd_ithvar(0, i) & fdd_ithvar(1, i));
		   }

		   /* Creating bddErrors*/
		   populatingErrorDS();
		   
        }


        void WpdsHandler::handleRule(
                const XMLCh* const uri
                , const XMLCh* const localname
                , const XMLCh* const qname
                , const Attributes& attributes)
        {
		  from = attributes.getValue(fromID);
		  fromStack = attributes.getValue(fromStackID);
		  to = attributes.getValue(toID);
		  toStack1 = attributes.getValue(toStack1ID);
		  toStack2 = attributes.getValue(toStack2ID);
		  
		  function = fromStack.get();
		  string::size_type loc = function.find(".", 0);
		  if (loc != string::npos) {
			function = function.substr(0, loc);
		  }

		  Key toStk1Key = getKey(toStack1.get());
		  Key toStk2Key = getKey(toStack2.get());
		  
		  if ( (toStk1Key != WALI_EPSILON) && (toStk2Key != WALI_EPSILON)) {
			callee = toStack1.get();
			string::size_type loc = callee.find(".", 0);
			if (loc != string::npos) {
			  callee = callee.substr(0, loc);
			}
		  }
		  else {
			callee = "";
		  }
		  
        }


	  void WpdsHandler::handleSet(								
                const XMLCh* const uri
                , const XMLCh* const localname
                , const XMLCh* const qname
                , const Attributes& attributes)
        {
            loc1 = attributes.getValue(loc1ID);
		   trusted = attributes.getValue(trustedID);
        }


	  void WpdsHandler::handleFrom(
                const XMLCh* const uri
                , const XMLCh* const localname
                , const XMLCh* const qname
                , const Attributes& attributes)
        {
            loc2 = attributes.getValue(loc2ID);
        }


	  void WpdsHandler::handleSource(
                const XMLCh* const uri
                , const XMLCh* const localname
                , const XMLCh* const qname
                , const Attributes& attributes)
        {
		  sline = attributes.getValue(lineID);
		  sscanf(sline.localForm(), "%d", &line);

		  sfile = attributes.getValue(fileID);
		  file = sfile.localForm();
        }


    } // namespace wpds

} // namespace wali


void populatingErrorDS() {

  for(unsigned int i = 0; i < errorNames.size(); i++) {
    int index = variables->getIndex(errorNames[i]);
    bddErrors |= fdd_ithvar(1, index);

    if (i < 34) {
	  int indexTentative = variables->getIndex("TENTATIVE_" + errorNames[i]);
	  bddTentativeErrors |= fdd_ithvar(1, indexTentative);
	  
	  mapTentativeToReal[indexTentative] = index;
	  //mapTentativeToReal[index] = indexTentative;
	  mapRealToTentative[index] = indexTentative;
	}
  }


  bddErrorsOK = bddErrors | fdd_ithvar(1, variables->getIndex("OK"));
  
  for(unsigned int i = 0; i < errorNames.size(); i++) {
    int index = variables->getIndex(errorNames[i]);
    bdd result = bddErrorsOK - fdd_ithvar(1, index);
    mapErrorToBdd[index] = result;
  }
}


/* Yo, Emacs!
   ;;; Local Variables: ***
   ;;; tab-width: 4 ***
   ;;; End: ***
 */
