/*!
 * @author Nick Kidd, Modified by Cindy Rubio Gonzalez
 */

#include "wali/Common.hpp"
#include "wali/wpds/ewpds/EWpdsHandler.hpp"
#include "StrX.hpp"
#include "wali/WeightFactory.hpp"
#include <xercesc/sax2/Attributes.hpp>
#include "wali/Key.hpp"
#include "wali/wpds/fwpds/FWPDS.hpp"
#include "wali/wpds/Rule.hpp"
#include <map>
#include <queue>
#include <vector>
#include "../../error-intra/ProgramPoint.hpp"

using namespace std;

typedef vector<ProgramPoint> ProgramPoints;
ProgramPoints programPoints;


extern queue<WeightInfo> weightInfos;
extern map<string, string> targets;

namespace wali
{
    namespace wpds
    {
        namespace ewpds
        {

            EWpdsHandler::EWpdsHandler( wali::wpds::fwpds::FWPDS* ep, WeightFactory& wf, MergeFactory& mf ) :
                epds(ep),
                WpdsHandler(wf,ep),
                mergeFactory(mf),
                inMerge(false),
                mergeString("")
            {
            }

            EWpdsHandler::~EWpdsHandler()
            {}

            wali::wpds::fwpds::FWPDS& EWpdsHandler::get_ewpds() {
                { // BEGIN DEBUGGING
                    assert(epds);
                } // END DEBUGGING
                return *epds;
            }

            //////////////////////////////////////////////////
            // Parsing handlers overriddend by EWpdsHandler
            //////////////////////////////////////////////////

            void EWpdsHandler::startElement(  const   XMLCh* const    uri,
                    const   XMLCh* const    localname,
                    const   XMLCh* const    qname,
                    const   Attributes&     attributes)
            {

                StrX who(localname);
                if( MergeFn::XMLTag == who.get() ) {
                    inMerge = true;
                    mergeString = "";
                }
                else {
                    WpdsHandler::startElement(uri,localname,qname,attributes);
                }
            }

            void EWpdsHandler::endElement( const XMLCh* const uri,
                    const XMLCh* const localname,
                    const XMLCh* const qname)
            {
                using wali::Key;
                StrX who(localname);
                if( MergeFn::XMLTag == who.get() ) {
                    inMerge = false;
                }
                else if( Rule::XMLTag == who.get() ) {
				  Key fromKey = getKey(from.get());
				  Key fromStkKey = getKey(fromStack.get());
				  Key toKey = getKey(to.get());
				  Key toStk1Key = getKey(toStack1.get());
				  Key toStk2Key = getKey(toStack2.get());

				  // delete later
				  string fstk = fromStack.get();
				  string::size_type loc = fstk.find("first", 0);
				  if (loc != string::npos) {
					std::cerr << "+ Function: " << fstk << std::endl;
				  }
				  // delete until here


				  string msgcode = "overwrite";
				  if ( (toStk1Key == WALI_EPSILON) && (toStk2Key == WALI_EPSILON) ) {
					// Pop Rule
					msgcode = "outOfScope";
					ErrorPropagationPopRule wf;
					get_ewpds().add_rule(fromKey
										 , fromStkKey
										 , toKey
										 , toStk1Key
										 , toStk2Key
										 /*, weightFactory.getWeight(weightInfos, line, file)*/
										 , wf.getWeight(weightInfos, line, file)
										 );

				  }

				  else if( toStk2Key == WALI_EPSILON ) {
					// neither pop nor push rules, use regular weight factory
					get_ewpds().add_rule(fromKey
										 , fromStkKey
										 , toKey
										 , toStk1Key
										 , toStk2Key
										 , weightFactory.getWeight(weightInfos, line, file)
										 );
				  }
				  else {
					// push rule
					ErrorPropagationPushRule wf;
					get_ewpds().add_rule(fromKey
										 , fromStkKey
										 , toKey
										 , toStk1Key
										 , toStk2Key
										 /*, weightFactory.getWeight(weightInfos, line, file)*/
										 , wf.getWeight(weightInfos, line, file)
										 /*, mergeFactory.getMergeFn(mergeString)*/
										 , mergeFactory.getMergeFn(callee)
										 );

				  }


				  if (inSet) {
					ProgramPoint point(file, function, line, targets, msgcode, fromStkKey);
					programPoints.push_back(point);
				  }

				  targets.clear();
				  
				  inSet = false;
				  inIdentity = false;
				  
				  
                }
                else {
				  WpdsHandler::endElement(uri,localname,qname);
                }
            }

            void EWpdsHandler::characters(const XMLCh* const chars, const unsigned int length)
            {
                StrX part(chars);
                if( inMerge ) {
                    assert( !inWeight );
                    mergeString += part.get();
                }
                else {
                    WpdsHandler::characters(chars,length);
                }
            }

        } // namespace ewpds

    } // namespace wpds

} // namespace wali



/* Yo, Emacs!
   ;;; Local Variables: ***
   ;;; tab-width: 4 ***
   ;;; End: ***
   */
