open Cil
open Printf


type receiver = varinfo

(*0 and 1 in mli*)
type sender = OK | Code of Error.code | Location of varinfo

type trust = Trusted | Untrusted


let rec getReceiver = function
  | Var varinfo, _ -> Some varinfo
  | Mem Lval lvalue, _ -> getReceiver lvalue
  | Mem _, _ -> None


let rec getSenderLval = function
  | Var varinfo, _ -> Some (Location varinfo)
  | Mem Lval lvalue, _ -> getSenderLval lvalue
  | Mem _, _ -> None


(*2*)
let rec getSender = function
  | CastE (_, expr) -> getSender expr
  | Lval lvalue -> getSenderLval lvalue
  (*| Const (CInt64 (-811824149L, _, _)) -> Some Error*)
  | Const (CInt64 (value, _, _)) ->
      begin
	match Error.byValue value with
	| Some code -> Some (Code code)
	| None -> Some OK
      end
  | Const _
  | SizeOf _
  | SizeOfStr _
  | AlignOf _
  | SizeOfE _
  | AlignOfE _
  | UnOp _
  | BinOp _
  | AddrOf _
  | StartOf _
    ->
      Some OK


let receiverText receiver = receiver.vname

(*3 and 4*)
let senderText =
  let reserved () =
    Errormsg.s (Errormsg.bug "location with reserved name")
  in
  function
    | OK -> "OK"
    | Code code -> Error.name code
    | Location {vname = "OK"} -> reserved ()
    | Location {vname = vname} ->
	match Error.byName vname with
	| Some _ -> reserved ()
	| None -> vname


let trustText = function
  | Trusted -> "true"
  | Untrusted -> "false"


let set sink receiver sender trusted =
  ignore (fprintf sink "\n\t\t\t<set loc=\"%s\" trusted=\"%s\">\n" (receiverText receiver) (trustText trusted));
  ignore (fprintf sink "\t\t\t\t<from loc=\"%s\"/>\n" (senderText sender));
  ignore (fprintf sink "\t\t\t</set>\n")


let unimplemented sink reason =
  ignore (fprintf sink "\n\t\t\t<unimplemented reason=\"%s\"/>\n" reason)
