from SCons.Builder import Builder
from SCons.Scanner import Scanner


########################################################################
#
#  generate a weighted pushdown system from a C source file
#

wpds_action = [
    ['$CILLY', '--dowpds', '--queryfile', '$TARGET', '--dtd', '$WPDS_DTD.abspath', '$__WPDS_MAIN_FUNCTION_FILE', '--error-codes', '$ERROR_CODES', '$SOURCES'],
    ['xmllint', '--noout', '--valid', '$TARGET'],
    ]


def wpds_target_scanner(node, env, path):
    deps = [env['CILLY'], env['WPDS_DTD'], env['ERROR_CODES']]
    if env['WPDS_MAIN_FUNCTION_FILE']:
        deps.append(env['WPDS_MAIN_FUNCTION_FILE'])
    return deps


wpds_builder = Builder(
    action=wpds_action,
    suffix='.wpds',
    src_suffix='.i',
    src_builder='Preprocess',
    target_scanner=Scanner(wpds_target_scanner),
    )


def wpds_main_function_file(target, source, env, for_signature):
    funcfile = env['WPDS_MAIN_FUNCTION_FILE']
    if funcfile:
        return ['--functionfile', env.File(funcfile).abspath]
    else:
        return []


########################################################################


def generate(env):
    env.AppendUnique(
        BUILDERS={'WPDS': wpds_builder},
        WPDS_DTD=env.File('#wpds-xml/dtd-files/wpds.dtd'),
        WPDS_MAIN_FUNCTION_FILE='',
        __WPDS_MAIN_FUNCTION_FILE=wpds_main_function_file,
        )


def exists(env):
    return True
